@@ -35,7 +35,7 @@ pub type SyncHashMap<K, V> = SyncMapImpl<K, V>;
 ///
 /// The zero Map is empty and ready for use. A Map must not be copied after first use.
 pub struct SyncMapImpl<K, V> {
-    read: UnsafeCell<Map<K, *const V>>,
+    read: UnsafeCell<Map<K, V>>,
     dirty: Mutex<Map<K, V>>,
 }
 
@@ -45,6 +45,7 @@ unsafe impl<K, V> Send for SyncMapImpl<K, V> {}
 /// this is safety, dirty mutex ensure
 unsafe impl<K, V> Sync for SyncMapImpl<K, V> {}
 
+//TODO maybe K will use transmute_copy replace Clone?
 impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
     pub fn new_arc() -> Arc<Self> {
         Arc::new(Self::new())
@@ -64,6 +65,7 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
         }
     }
 
+
     pub fn insert(&self, k: K, v: V) -> Option<V> where K: Clone {
         match self.dirty.lock() {
             Ok(mut m) => {
@@ -72,7 +74,7 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
                     None => {
                         let r = m.get(&k);
                         unsafe {
-                            (&mut *self.read.get()).insert(k, r.unwrap());
+                            (&mut *self.read.get()).insert(k, std::mem::transmute_copy(r.unwrap()));
                         }
                         None
                     }
@@ -94,7 +96,13 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
                 match op {
                     Some(v) => {
                         unsafe {
-                            (&mut *self.read.get()).remove(k);
+                            let r=(&mut *self.read.get()).remove(k);
+                            match r{
+                                None => {}
+                                Some(r) => {
+                                    std::mem::forget(r);
+                                }
+                            }
                         }
                         Some(v)
                     }
@@ -124,10 +132,17 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
     pub fn clear(&self) {
         match self.dirty.lock() {
             Ok(mut m) => {
+                m.clear();
                 unsafe {
-                    (&mut *self.read.get()).clear()
+                    for x in (&*self.read.get()).keys() {
+                        match (&mut *self.read.get()).remove(x){
+                            None => {}
+                            Some(v) => {
+                                std::mem::forget(v);
+                            }
+                        }
+                    }
                 }
-                m.clear()
             }
             Err(_) => {}
         }
@@ -149,13 +164,10 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
         let s = Self::with_capacity(map.capacity());
         match s.dirty.lock() {
             Ok(mut m) => {
-                unsafe {
-                    (&mut *s.read.get()).clear();
-                }
                 *m = map;
                 unsafe {
                     for (k, v) in m.iter() {
-                        (&mut *s.read.get()).insert(k.clone(), v);
+                        (&mut *s.read.get()).insert(k.clone(), std::mem::transmute_copy(v));
                     }
                 }
             }
@@ -194,10 +206,7 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
             match k {
                 None => { None }
                 Some(s) => {
-                    if s.is_null() {
-                        return None;
-                    }
-                    Some(&**s)
+                    Some(s)
                 }
             }
         }
@@ -226,12 +235,9 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
         }
     }
 
-    pub fn iter(&self) -> Iter<'_, K, V> {
+    pub fn iter(&self) -> MapIter<'_, K, V> {
         unsafe {
-            let iter = (&*self.read.get()).iter();
-            Iter {
-                inner: Some(iter)
-            }
+             (&*self.read.get()).iter()
         }
     }
 
@@ -255,12 +261,9 @@ impl<K, V> SyncMapImpl<K, V> where K: std::cmp::Eq + Hash + Clone {
         }
     }
 
-    pub fn into_iter(self) -> Iter<'static, K, V> {
+    pub fn into_iter(self) -> MapIter<'static, K, V> {
         unsafe {
-            let iter = (&*self.read.get()).iter();
-            Iter {
-                inner: Some(iter)
-            }
+            (&*self.read.get()).iter()
         }
     }
 }
@@ -362,7 +365,7 @@ impl<'a, K, V> Iterator for IterMut<'a, K, V> {
 
 impl<'a, K, V> IntoIterator for &'a SyncMapImpl<K, V> where K: Eq + Hash + Clone {
     type Item = (&'a K, &'a V);
-    type IntoIter = Iter<'a, K, V>;
+    type IntoIter = MapIter<'a, K, V>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.iter()
@@ -383,7 +386,7 @@ impl<K, V> IntoIterator for SyncMapImpl<K, V> where
     K: Eq + Hash + Clone,
     K: 'static, V: 'static {
     type Item = (&'static K, &'static V);
-    type IntoIter = Iter<'static, K, V>;
+    type IntoIter = MapIter<'static, K, V>;
 
     fn into_iter(mut self) -> Self::IntoIter {
         self.into_iter()
